{% include 'header.html' %}
{% include 'navbar.html' %}
<h1>Welcome to the {{pdk_name}} pdk</h1>
<p>
{% for component in components %}
<a class="component-link" href="/view/{{component}}">{{component}}</a>
{% endfor %}
</p>

{% include 'footer.html' %}
